@@ -1,12 +1,13 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (c) 2012-2014 Monty Program Ab
-// Copyright (c) 2015-2023 MariaDB Corporation Ab
+// Copyright (c) 2015-2025 MariaDB Corporation Ab
 package org.mariadb.jdbc.client.column;
 
 import java.sql.*;
 import java.util.Calendar;
 import org.mariadb.jdbc.Configuration;
 import org.mariadb.jdbc.client.ColumnDecoder;
+import org.mariadb.jdbc.client.Context;
 import org.mariadb.jdbc.client.DataType;
 import org.mariadb.jdbc.client.ReadableByteBuf;
 import org.mariadb.jdbc.client.util.MutableInt;
@@ -29,15 +30,15 @@ public class SignedIntColumn extends ColumnDefinitionPacket implements ColumnDec
    * @param extTypeFormat extended type format
    */
   public SignedIntColumn(
-      ReadableByteBuf buf,
-      int charset,
-      long length,
-      DataType dataType,
-      byte decimals,
-      int flags,
-      int[] stringPos,
-      String extTypeName,
-      String extTypeFormat) {
+      final ReadableByteBuf buf,
+      final int charset,
+      final long length,
+      final DataType dataType,
+      final byte decimals,
+      final int flags,
+      final int[] stringPos,
+      final String extTypeName,
+      final String extTypeFormat) {
     super(
         buf,
         charset,
@@ -51,53 +52,69 @@ public class SignedIntColumn extends ColumnDefinitionPacket implements ColumnDec
         false);
   }
 
+  /**
+   * Recreate new column using alias as name.
+   *
+   * @param prev current column
+   */
   protected SignedIntColumn(SignedIntColumn prev) {
     super(prev, true);
   }
 
+  public int getPrecision() {
+    // UNSIGNED INT :             0..4294967295 digits=10 nchars=10
+    // SIGNED INT   :   -2147483648..2147483647 digits=10 nchars=11
+    // display size is correct, but need to limit to 10 for precision
+    return Math.min(10, (int) columnLength);
+  }
+
   @Override
   public SignedIntColumn useAliasAsName() {
     return new SignedIntColumn(this);
   }
 
-  public String defaultClassname(Configuration conf) {
+  public String defaultClassname(final Configuration conf) {
     return Integer.class.getName();
   }
 
-  public int getColumnType(Configuration conf) {
+  public int getColumnType(final Configuration conf) {
     return Types.INTEGER;
   }
 
-  public String getColumnTypeName(Configuration conf) {
+  public String getColumnTypeName(final Configuration conf) {
     return "INTEGER";
   }
 
   @Override
-  public Object getDefaultText(final Configuration conf, ReadableByteBuf buf, MutableInt length)
+  public Object getDefaultText(
+      final ReadableByteBuf buf, final MutableInt length, final Context context)
       throws SQLDataException {
     return (int) buf.atoll(length.get());
   }
 
   @Override
-  public Object getDefaultBinary(final Configuration conf, ReadableByteBuf buf, MutableInt length)
+  public Object getDefaultBinary(
+      final ReadableByteBuf buf, final MutableInt length, final Context context)
       throws SQLDataException {
     return buf.readInt();
   }
 
   @Override
-  public boolean decodeBooleanText(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public boolean decodeBooleanText(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     String s = buf.readAscii(length.get());
     return !"0".equals(s);
   }
 
   @Override
-  public boolean decodeBooleanBinary(ReadableByteBuf buf, MutableInt length)
+  public boolean decodeBooleanBinary(final ReadableByteBuf buf, final MutableInt length)
       throws SQLDataException {
     return buf.readInt() != 0;
   }
 
   @Override
-  public byte decodeByteText(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public byte decodeByteText(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     long result = buf.atoll(length.get());
     if ((byte) result != result || (result < 0 && !isSigned())) {
       throw new SQLDataException("byte overflow");
@@ -106,7 +123,8 @@ public class SignedIntColumn extends ColumnDefinitionPacket implements ColumnDec
   }
 
   @Override
-  public byte decodeByteBinary(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public byte decodeByteBinary(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     long result = buf.readInt();
     if ((byte) result != result) {
       throw new SQLDataException("byte overflow");
@@ -116,19 +134,22 @@ public class SignedIntColumn extends ColumnDefinitionPacket implements ColumnDec
   }
 
   @Override
-  public String decodeStringText(ReadableByteBuf buf, MutableInt length, Calendar cal)
+  public String decodeStringText(
+      final ReadableByteBuf buf, final MutableInt length, final Calendar cal, final Context context)
       throws SQLDataException {
     return buf.readString(length.get());
   }
 
   @Override
-  public String decodeStringBinary(ReadableByteBuf buf, MutableInt length, Calendar cal)
+  public String decodeStringBinary(
+      final ReadableByteBuf buf, final MutableInt length, final Calendar cal, final Context context)
       throws SQLDataException {
     return String.valueOf(buf.readInt());
   }
 
   @Override
-  public short decodeShortText(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public short decodeShortText(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     long result = buf.atoll(length.get());
     if ((short) result != result) {
       throw new SQLDataException("Short overflow");
@@ -137,7 +158,8 @@ public class SignedIntColumn extends ColumnDefinitionPacket implements ColumnDec
   }
 
   @Override
-  public short decodeShortBinary(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public short decodeShortBinary(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     long result = isSigned() ? buf.readInt() : buf.readUnsignedInt();
     if ((short) result != result) {
       throw new SQLDataException("Short overflow");
@@ -146,75 +168,88 @@ public class SignedIntColumn extends ColumnDefinitionPacket implements ColumnDec
   }
 
   @Override
-  public int decodeIntText(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public int decodeIntText(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     return (int) buf.atoll(length.get());
   }
 
   @Override
-  public int decodeIntBinary(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public int decodeIntBinary(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     return buf.readInt();
   }
 
   @Override
-  public long decodeLongText(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public long decodeLongText(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     return buf.atoll(length.get());
   }
 
   @Override
-  public long decodeLongBinary(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public long decodeLongBinary(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     return buf.readInt();
   }
 
   @Override
-  public float decodeFloatText(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public float decodeFloatText(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     return Float.parseFloat(buf.readAscii(length.get()));
   }
 
   @Override
-  public float decodeFloatBinary(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public float decodeFloatBinary(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     return (float) buf.readInt();
   }
 
   @Override
-  public double decodeDoubleText(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public double decodeDoubleText(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     return Double.parseDouble(buf.readAscii(length.get()));
   }
 
   @Override
-  public double decodeDoubleBinary(ReadableByteBuf buf, MutableInt length) throws SQLDataException {
+  public double decodeDoubleBinary(final ReadableByteBuf buf, final MutableInt length)
+      throws SQLDataException {
     return buf.readInt();
   }
 
   @Override
-  public Date decodeDateText(ReadableByteBuf buf, MutableInt length, Calendar cal)
+  public Date decodeDateText(
+      final ReadableByteBuf buf, final MutableInt length, final Calendar cal, final Context context)
       throws SQLDataException {
     buf.skip(length.get());
     throw new SQLDataException(String.format("Data type %s cannot be decoded as Date", dataType));
   }
 
   @Override
-  public Date decodeDateBinary(ReadableByteBuf buf, MutableInt length, Calendar cal)
+  public Date decodeDateBinary(
+      final ReadableByteBuf buf, final MutableInt length, final Calendar cal, final Context context)
       throws SQLDataException {
     buf.skip(length.get());
     throw new SQLDataException(String.format("Data type %s cannot be decoded as Date", dataType));
   }
 
   @Override
-  public Time decodeTimeText(ReadableByteBuf buf, MutableInt length, Calendar cal)
+  public Time decodeTimeText(
+      final ReadableByteBuf buf, final MutableInt length, final Calendar cal, final Context context)
       throws SQLDataException {
     buf.skip(length.get());
     throw new SQLDataException(String.format("Data type %s cannot be decoded as Time", dataType));
   }
 
   @Override
-  public Time decodeTimeBinary(ReadableByteBuf buf, MutableInt length, Calendar cal)
+  public Time decodeTimeBinary(
+      final ReadableByteBuf buf, final MutableInt length, final Calendar cal, final Context context)
       throws SQLDataException {
     buf.skip(length.get());
     throw new SQLDataException(String.format("Data type %s cannot be decoded as Time", dataType));
   }
 
   @Override
-  public Timestamp decodeTimestampText(ReadableByteBuf buf, MutableInt length, Calendar cal)
+  public Timestamp decodeTimestampText(
+      final ReadableByteBuf buf, final MutableInt length, Calendar cal, final Context context)
       throws SQLDataException {
     buf.skip(length.get());
     throw new SQLDataException(
@@ -222,7 +257,8 @@ public class SignedIntColumn extends ColumnDefinitionPacket implements ColumnDec
   }
 
   @Override
-  public Timestamp decodeTimestampBinary(ReadableByteBuf buf, MutableInt length, Calendar cal)
+  public Timestamp decodeTimestampBinary(
+      final ReadableByteBuf buf, final MutableInt length, Calendar cal, final Context context)
       throws SQLDataException {
     buf.skip(length.get());
     throw new SQLDataException(
