@@ -7,7 +7,12 @@ import static org.mariadb.jdbc.client.impl.ConnectionHelper.enabledSslCipherSuit
 import static org.mariadb.jdbc.client.impl.ConnectionHelper.enabledSslProtocolSuites;
 import static org.mariadb.jdbc.util.constants.Capabilities.SSL;
 
-import java.io.*;
+import java.io.BufferedInputStream;
+import java.io.BufferedOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.io.UnsupportedEncodingException;
 import java.net.Socket;
 import java.net.SocketException;
 import java.net.SocketTimeoutException;
@@ -24,7 +29,11 @@ import java.time.DateTimeException;
 import java.time.Instant;
 import java.time.ZoneId;
 import java.time.ZoneOffset;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.List;
+import java.util.TimeZone;
 import java.util.concurrent.Executor;
 import java.util.function.Consumer;
 import java.util.regex.Matcher;
@@ -43,7 +52,12 @@ import org.mariadb.jdbc.client.result.Result;
 import org.mariadb.jdbc.client.result.StreamingResult;
 import org.mariadb.jdbc.client.socket.Reader;
 import org.mariadb.jdbc.client.socket.Writer;
-import org.mariadb.jdbc.client.socket.impl.*;
+import org.mariadb.jdbc.client.socket.impl.CompressInputStream;
+import org.mariadb.jdbc.client.socket.impl.CompressOutputStream;
+import org.mariadb.jdbc.client.socket.impl.PacketReader;
+import org.mariadb.jdbc.client.socket.impl.PacketWriter;
+import org.mariadb.jdbc.client.socket.impl.ReadAheadBufferedStream;
+import org.mariadb.jdbc.client.socket.impl.UnixDomainSocket;
 import org.mariadb.jdbc.client.tls.MariaDbX509EphemeralTrustingManager;
 import org.mariadb.jdbc.client.util.ClosableLock;
 import org.mariadb.jdbc.client.util.MutableByte;
@@ -52,13 +66,26 @@ import org.mariadb.jdbc.export.MaxAllowedPacketException;
 import org.mariadb.jdbc.export.Prepare;
 import org.mariadb.jdbc.export.SslMode;
 import org.mariadb.jdbc.message.ClientMessage;
-import org.mariadb.jdbc.message.client.*;
-import org.mariadb.jdbc.message.server.*;
-import org.mariadb.jdbc.plugin.*;
+import org.mariadb.jdbc.message.client.ClosePreparePacket;
+import org.mariadb.jdbc.message.client.HandshakeResponse;
+import org.mariadb.jdbc.message.client.QueryPacket;
+import org.mariadb.jdbc.message.client.QuitPacket;
+import org.mariadb.jdbc.message.client.SslRequestPacket;
+import org.mariadb.jdbc.message.server.AuthSwitchPacket;
+import org.mariadb.jdbc.message.server.ErrorPacket;
+import org.mariadb.jdbc.message.server.InitialHandshakePacket;
+import org.mariadb.jdbc.message.server.OkPacket;
+import org.mariadb.jdbc.message.server.PrepareResultPacket;
+import org.mariadb.jdbc.plugin.AuthenticationPlugin;
+import org.mariadb.jdbc.plugin.AuthenticationPluginFactory;
+import org.mariadb.jdbc.plugin.Credential;
+import org.mariadb.jdbc.plugin.CredentialPlugin;
+import org.mariadb.jdbc.plugin.TlsSocketPlugin;
 import org.mariadb.jdbc.plugin.authentication.AuthenticationPluginLoader;
 import org.mariadb.jdbc.plugin.authentication.addon.ClearPasswordPlugin;
 import org.mariadb.jdbc.plugin.authentication.standard.NativePasswordPlugin;
 import org.mariadb.jdbc.plugin.tls.TlsSocketPluginLoader;
+import org.mariadb.jdbc.util.IPUtility;
 import org.mariadb.jdbc.util.Security;
 import org.mariadb.jdbc.util.StringUtils;
 import org.mariadb.jdbc.util.constants.Capabilities;
@@ -257,6 +284,14 @@ public class StandardClient implements Client, AutoCloseable {
 
   private void handleSslHandshake(SSLSocket sslSocket, TrustManager[] trustManagers)
       throws IOException {
+    // Set SNI hostname
+    if (this.hostAddress != null && !IPUtility.isInetAddress(this.hostAddress.host)) {
+      SSLParameters params = sslSocket.getSSLParameters();
+      SNIHostName serverName = new SNIHostName(this.hostAddress.host);
+      params.setServerNames(Collections.singletonList(serverName));
+      sslSocket.setSSLParameters(params);
+    }
+
     sslSocket.startHandshake();
     if (trustManagers.length > 0
         && trustManagers[0] instanceof MariaDbX509EphemeralTrustingManager) {
@@ -265,7 +300,7 @@ public class StandardClient implements Client, AutoCloseable {
   }
 
   private boolean requiresHostnameVerification(SslMode sslMode) {
-    return certFingerprint == null && sslMode == SslMode.VERIFY_FULL && hostAddress.host != null;
+    return sslMode == SslMode.VERIFY_FULL && certFingerprint == null && hostAddress.host != null;
   }
 
   private void verifyHostname(SSLSocket sslSocket, TlsSocketPlugin socketPlugin)
@@ -406,7 +441,7 @@ public class StandardClient implements Client, AutoCloseable {
       InitialHandshakePacket handshake, Credential credential, String authType) {
     authPlugin =
         "mysql_clear_password".equals(authType)
-            ? new ClearPasswordPlugin(credential.getPassword())
+            ? new ClearPasswordPlugin(credential.getPassword(), hostAddress, conf)
             : new NativePasswordPlugin(credential.getPassword(), handshake.getSeed());
   }
 
@@ -461,7 +496,7 @@ public class StandardClient implements Client, AutoCloseable {
                         authPluginFactory.type()));
           }
 
-          buf = authPlugin.process(writer, reader, context);
+          buf = authPlugin.process(writer, reader, context, certFingerprint != null);
           break;
 
         case 0xFF:
@@ -714,7 +749,7 @@ public class StandardClient implements Client, AutoCloseable {
   }
 
   private void postConnectionQueries() throws SQLException {
-    List<String> commands = new ArrayList<>();
+    List<String> commands = new ArrayList<>(8); // typical: timezone, session vars, init sql
 
     List<String> galeraAllowedStates =
         conf.galeraAllowedState() == null
@@ -738,12 +773,8 @@ public class StandardClient implements Client, AutoCloseable {
       commands.add(String.format("USE `%s`", escapedDb));
     }
 
-    if (conf.initSql() != null) {
-      commands.add(conf.initSql());
-    }
-
-    if (conf.nonMappedOptions().containsKey("initSql")) {
-      String[] initialCommands = conf.nonMappedOptions().get("initSql").toString().split(";");
+    if (conf.initSql() != null && !conf.initSql().isEmpty()) {
+      String[] initialCommands = conf.initSql().split(";");
       Collections.addAll(commands, initialCommands);
     }
 
@@ -822,7 +853,7 @@ public class StandardClient implements Client, AutoCloseable {
    * @return query string for setting session variables or null if no variables need to be set
    */
   public String createSessionVariableQuery(Context context) {
-    List<String> sessionCommands = new ArrayList<>();
+    List<String> sessionCommands = new ArrayList<>(8); // autocommit, truncation, tracking, etc.
 
     addAutoCommitCommand(context, sessionCommands);
     addTruncationCommand(sessionCommands);
@@ -906,7 +937,7 @@ public class StandardClient implements Client, AutoCloseable {
     ZoneId connectionZoneId = connectionTz.toZoneId();
 
     if ("SERVER".equalsIgnoreCase(conf.connectionTimeZone())
-            && connectionZoneId.normalized().equals(TimeZone.getDefault().toZoneId())) {
+        && connectionZoneId.normalized().equals(TimeZone.getDefault().toZoneId())) {
       return;
     }
 
@@ -1049,7 +1080,7 @@ public class StandardClient implements Client, AutoCloseable {
       boolean closeOnCompletion,
       boolean canRedo)
       throws SQLException {
-    List<Completion> results = new ArrayList<>();
+    List<Completion> results = new ArrayList<>(messages.length); // one result per message
     int perMsgCounter = 0;
     int readCounter = 0;
     int[] responseMsg = new int[messages.length];
@@ -1092,7 +1123,7 @@ public class StandardClient implements Client, AutoCloseable {
         results.add(null);
         // read remaining results
         perMsgCounter++;
-        for (; perMsgCounter < responseMsg[readCounter - 1]; perMsgCounter++) {
+        for (; readCounter > 0 && perMsgCounter < responseMsg[readCounter - 1]; perMsgCounter++) {
           try {
             results.addAll(
                 readResponse(
@@ -1172,7 +1203,7 @@ public class StandardClient implements Client, AutoCloseable {
         streamStmt.fetchRemaining();
         streamStmt = null;
       }
-      List<Completion> completions = new ArrayList<>();
+      List<Completion> completions = new ArrayList<>(1); // typically single completion
       try {
         while (nbResp-- > 0) {
           readResults(
@@ -1234,7 +1265,7 @@ public class StandardClient implements Client, AutoCloseable {
       streamStmt.fetchRemaining();
       streamStmt = null;
     }
-    List<Completion> completions = new ArrayList<>();
+    List<Completion> completions = new ArrayList<>(1); // single message execution
     readResults(
         stmt,
         message,
@@ -1259,7 +1290,7 @@ public class StandardClient implements Client, AutoCloseable {
       streamStmt.fetchRemaining();
       streamStmt = null;
     }
-    List<Completion> completions = new ArrayList<>();
+    List<Completion> completions = new ArrayList<>(1); // single message execution
     readResults(
         null,
         message,
