@@ -8,12 +8,15 @@ import static org.junit.jupiter.api.Assertions.*;
 import java.sql.*;
 import java.sql.Connection;
 import java.sql.Statement;
+import java.time.Duration;
+import java.time.Instant;
 import java.util.*;
 import java.util.concurrent.Executor;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import org.junit.jupiter.api.*;
 import org.mariadb.jdbc.*;
+import org.mariadb.jdbc.export.HaMode;
 import org.mariadb.jdbc.integration.util.SocketFactoryBasicTest;
 import org.mariadb.jdbc.integration.util.SocketFactoryTest;
 import org.mariadb.jdbc.util.constants.Capabilities;
@@ -114,6 +117,13 @@ public class ConnectionTest extends Common {
       assertTrue(rs.next());
       assertEquals("YourVar", rs.getString(1));
     }
+    try (Connection con = createCon("&initSql=SET @myVar='YourVar';SET @myVar2='YourVar2'")) {
+      Statement stmt = con.createStatement();
+      ResultSet rs = stmt.executeQuery("SELECT @myVar, @myVar2");
+      assertTrue(rs.next());
+      assertEquals("YourVar", rs.getString(1));
+      assertEquals("YourVar2", rs.getString(2));
+    }
   }
 
   @Test
@@ -641,6 +651,16 @@ public class ConnectionTest extends Common {
     }
   }
 
+  @Test
+  public void savepointNotExistingTest() throws SQLException {
+    try (Connection con = createCon()) {
+      Savepoint sp = con.setSavepoint("savepoint1");
+      con.setAutoCommit(false);
+      con.commit();
+      assertThrows(SQLSyntaxErrorException.class, () -> con.rollback(sp));
+    }
+  }
+
   @Test
   public void netWorkTimeout() throws SQLException {
     Connection con = createCon();
@@ -796,11 +816,7 @@ public class ConnectionTest extends Common {
     stmt.execute("GRANT SELECT on `" + database + "`.* to verifParsec" + getHostSuffix());
     stmt.execute("GRANT SELECT on `" + database + "`.* to verifParsec2" + getHostSuffix());
 
-    String version = System.getProperty("java.version");
-    int majorVersion =
-        (version.indexOf(".") >= 0)
-            ? Integer.parseInt(version.substring(0, version.indexOf(".")))
-            : Integer.parseInt(version);
+    int majorVersion = getJavaVersion();
     if (majorVersion < 15) {
       // before java 15, Ed25519 is not supported
       // assuming, that BouncyCastle is not on test classpath
@@ -1492,4 +1508,23 @@ public class ConnectionTest extends Common {
     assertTrue(stmt.isClosed());
     assertTrue(preparedStatement.isClosed());
   }
+
+  @Test
+  public void isValidTimeoutValidation() throws SQLException {
+    try (org.mariadb.jdbc.Connection con = createProxyCon(HaMode.NONE, "")) {
+      proxy.setDelay(2000);
+      Instant start = Instant.now();
+      try {
+        con.isValid(1);
+      } catch (SQLException e) {
+        // eat
+      }
+      Instant end = Instant.now();
+      Duration duration = Duration.between(start, end);
+      assertTrue(duration.toMillis() > 950);
+      assertTrue(duration.toMillis() < 1500);
+    } finally {
+      proxy.forceClose();
+    }
+  }
 }
